
# q-expansion of newform 7800.2.a.bi, downloaded from the LMFDB on 07 May 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField, ZZ
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K([c/ZZ(den) for c in num]) for num, den in basis_data]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 7800
weight = 2
poly_data = [-4, -7, 0, 1]

# The entries in the following list give a basis for the
# coefficient ring in terms of a root of the defining polynomial above.
# Each line consists of the coefficients of the numerator, and a denominator.
basis_data  = [[[1, 0, 0], 1], [[-5, 0, 1], 1], [[5, 2, -1], 1]]

hecke_ring_character_values = None
aps_data = [[0, 0, 0], [-1, 0, 0], [0, 0, 0], [0, 0, -1], [2, 1, 0], [1, 0, 0], [-2, 2, 1], [2, 1, 1], [0, 0, 1], [4, 1, -1], [0, -2, 0], [-4, -1, 0], [0, 1, 0], [0, 2, 2], [4, 2, 0], [0, 1, -2], [4, 2, -2], [0, 1, -2], [4, -2, -2], [-6, 1, 0], [-8, 1, -1], [6, -1, 0], [0, -2, 2], [4, -1, 0], [-2, 0, -3], [-12, -3, 1], [8, -2, -2], [2, 3, 2], [8, 1, 1], [0, -1, 1], [4, -4, -2], [-2, -1, 3], [-2, 0, 0], [6, -3, 0], [4, -1, 0], [-4, 0, -4], [-2, 4, 0], [2, 1, 2], [12, -2, -2], [6, 0, 0], [-6, 1, -3], [4, -1, -4], [0, 0, -4], [-10, -4, -1], [-2, -2, 4], [12, 2, 2], [-4, 4, 4], [18, 1, -1], [0, 2, 0], [4, -1, 1], [2, 4, 1], [-2, 3, -2], [2, -4, -4], [6, 3, 5], [8, 3, 5], [14, -1, 1], [-12, 1, 1], [0, -6, -4], [-2, -4, 4], [-14, -2, 2], [16, 2, -2], [6, -2, 0], [6, 3, 0], [-16, -4, 0], [-22, -2, -2], [14, -2, 0], [-10, 3, 1], [14, -4, -2], [-10, -3, -4], [0, 1, 5], [14, 0, 0], [-12, 2, -2], [0, 2, 2], [10, -2, -4], [-2, -1, -5], [16, 0, 2], [-12, -3, 1], [12, -5, -4], [-6, -2, -6], [14, -2, -2], [6, -5, -1], [-20, -5, 3], [8, 4, -4], [-10, 0, 2], [6, -5, -6], [-10, 1, 4], [16, 1, 8], [-2, -4, -3], [-12, -1, -6], [4, 2, 5], [-6, -9, -2], [18, 5, -2], [20, 6, -1], [2, -7, -1], [-2, 3, 3], [14, 3, 1], [-16, 1, 2], [14, 2, 6], [4, 0, 0], [0, 5, -1], [4, 0, 4], [2, 4, -4], [-18, 1, 0], [18, 4, 6], [2, -1, 4], [10, 2, -1], [-24, 2, 0], [-2, -8, -2], [-8, -8, 0], [16, -5, 2], [-8, 2, -2], [-8, 5, 8], [-10, -8, -4], [14, 7, -1], [24, 2, 2], [-10, 2, 4], [30, -1, 0], [-8, 4, -1], [2, 6, -2], [-14, -3, 7], [-12, 7, 3], [-2, 4, 8], [-12, 11, 4], [-24, -6, -2], [-22, -7, 1], [-4, 1, 1], [0, 5, 1], [12, 2, -2], [8, 6, 4], [12, -9, -4], [6, -5, -5], [-8, -4, -6], [6, 3, 6], [-14, -6, -8], [18, 2, -6], [-2, -2, 4], [-10, -2, -4], [-8, -4, 0], [-20, 3, -2], [34, 0, -4], [14, 7, 1], [16, 1, -4], [0, -2, 2], [-4, 0, -6], [-18, 4, 4], [30, -5, -2], [-24, 5, 0], [6, -2, -1], [-6, -5, -2], [-8, 0, 2], [-10, -8, 8], [-18, -10, 6], [-4, 8, 4], [4, -6, -3], [28, -4, -2], [12, 2, -2], [34, 5, 2], [28, 3, 4], [-10, -8, 8], [-12, 3, 4], [0, 10, 2], [-18, -2, -3], [-2, -5, 5], [-2, -9, -7], [-10, 8, -2], [20, 2, 10], [26, 1, -4], [2, -6, -4], [2, -4, 8], [-16, -3, -10], [-16, 4, 8], [-4, 3, -3], [20, -6, 2], [-14, 2, -9], [-6, -15, 0], [-38, 4, 2], [-30, 1, -3], [12, 3, -4], [-44, 0, 0], [18, 2, 6], [-16, 0, -5], [10, 5, -1], [-10, -4, -12], [2, -10, -10], [16, 0, -2], [0, 11, 7], [18, 10, -4], [26, 1, -8], [-6, -4, -10], [18, 1, 6], [2, -2, 4], [-32, 2, 4], [-10, -7, 6], [8, 5, -8], [-8, -2, 10], [2, 8, -1], [-34, 6, -2], [-2, -12, 4], [30, 8, -2], [4, 6, -5], [-2, -8, -4], [50, 5, -2], [20, 11, -2], [-20, -11, 2], [-18, -5, 4], [40, 9, -4], [-24, -6, -4], [0, -2, -6], [4, -5, -4], [10, 3, -10], [30, 4, 10], [4, 5, 3], [-4, 0, 12], [-44, 4, 2], [-24, 12, 8], [-2, 2, 4], [12, 12, 4], [-26, -10, -2], [24, 12, -2], [-14, -4, 4], [24, -11, -8], [28, 12, -6], [10, -6, 2], [-10, -1, -3], [22, -11, -4], [-28, 3, -4], [10, 0, 5], [-48, 0, 2], [24, -2, -4], [2, 5, 8], [34, 6, 0], [18, -1, -2], [-12, 8, 8], [-14, 8, 6], [-12, 4, -10], [-32, -8, -6], [-14, 12, 2], [-54, -4, 0], [-22, -3, -11], [-36, -6, -2], [32, -6, -6], [26, -5, -6], [-12, 0, 12], [-28, 11, 7], [-10, -16, 2], [-24, 0, 4], [-30, 9, -1], [6, 3, 10], [-18, -5, -3], [-2, -1, 9], [26, 2, -6], [18, -2, 6], [12, 2, 4], [12, -3, 6], [12, -1, 6], [2, 1, -2], [6, -4, 12], [2, 1, -10], [-2, 4, -4], [-18, 12, 1], [-40, -10, 4], [6, 9, -10], [-8, -15, -7], [-22, 2, 8], [-6, -2, 2], [-22, -5, -8], [0, -3, 2], [-20, 11, 4], [2, 1, -4], [-14, 2, -6], [26, 14, 2], [-2, -13, -6], [-14, -14, 7], [24, 0, 0], [14, -12, 0], [28, 2, 7], [20, -8, -8], [-28, 3, 7], [42, -8, -2], [50, -3, -3], [-20, -6, -1], [-16, -10, 2], [38, 3, 5], [4, 11, 5], [-12, -14, -10], [4, 14, 2], [-34, 8, 10], [-42, 6, 4], [-20, 0, -10], [38, 6, 0], [-24, 3, 7], [-12, 0, 8], [-50, 8, 2], [8, -4, -8], [-18, 4, 8], [-8, 3, 9], [-12, 14, 2], [56, -3, -4], [-22, -7, 7], [-6, 13, 2], [18, 10, 6], [-42, -10, 0], [30, -1, 0], [10, -9, 8], [-4, -8, 4], [-58, -4, -1], [62, 1, -2], [-16, 13, 12], [16, 4, -4], [2, 6, 8], [8, -4, -3], [-10, 4, -8], [62, -2, 2], [-4, -8, -4], [-40, -8, 0], [-28, 1, 6], [30, -13, -4], [-46, -11, 8], [40, -3, -1], [22, 10, -4], [16, -14, -2], [8, 5, -5], [-8, -1, -3], [-12, 6, -3], [58, 6, -6], [24, 3, 6], [14, -17, -9], [38, -5, 0], [-36, 2, -13], [14, -16, -16], [48, 1, 7], [-8, -3, 16], [64, 4, -4], [12, 20, 4], [-18, -9, 9], [-8, 2, -12], [-56, -7, 3], [10, 14, -8], [14, -2, 14], [28, -20, -10], [4, -9, -2], [14, 18, -5], [-28, 11, 12], [60, -2, -2], [-6, 16, 8], [10, 1, 7], [-32, -7, -14], [-10, 11, 16], [28, -2, 10], [-64, 1, -8], [-2, -9, -17], [-2, -4, 15], [16, 5, 8], [-40, -10, 8], [-8, -7, 4], [4, -7, -5], [-2, 3, 6], [20, -2, 14], [-12, 1, 7], [-4, -18, 0], [-66, 0, 4], [-18, 10, 12], [22, -5, -7], [-8, 0, 4], [20, 12, -2], [-26, 0, 6], [-34, -6, -12], [-20, 10, 9], [4, -15, 6], [14, 11, 7], [-32, 18, 14], [-4, -2, 6], [12, -19, -5], [-38, 1, 12], [-62, 6, 0], [-46, -7, 2], [-4, -2, 8], [-6, -14, -1], [-30, -12, -2], [50, -10, -2], [10, 2, 6], [-10, -12, -8], [-54, 1, 7], [-6, -14, -18], [10, -11, -1], [-24, 0, -4], [-28, 4, -6], [18, -6, -4], [-28, -14, -2], [-12, 2, -9], [30, 18, -8], [50, 0, 12], [-30, 9, -4], [44, -12, 0], [2, -3, -4], [18, 6, -10], [16, -18, -4], [-38, -8, -4], [-30, 11, -2], [14, -20, -12], [-32, 5, -9], [-26, -6, 11], [8, -8, 5], [62, 4, 6], [-28, -9, 2], [6, -9, 14], [-46, 6, 2], [34, 6, -10], [6, -9, 2], [26, 13, 6], [18, -6, -2], [52, 3, 10], [14, -3, -8], [6, 9, 0], [-10, -12, -16], [20, -3, 3], [42, -3, -2], [20, 4, 2], [30, 0, 2], [-40, 4, 7], [20, -3, -5], [-72, 1, 2], [-40, -8, -12], [-34, -5, 9], [-8, 5, 19], [-10, 14, 4], [-6, -9, -14], [32, 9, 4], [-10, 3, 11], [-36, -2, 4], [-18, -18, -2], [-66, -1, 6], [18, 7, 0], [24, 4, 18], [16, 21, -8], [18, -4, -8], [-66, -2, -2], [20, 7, 9], [-64, -2, 2], [44, 4, 8], [4, 8, 8], [20, 7, 12], [-12, -13, -21], [-4, 6, -8], [-54, 0, 4], [48, 7, -7], [-56, 10, 8], [28, -6, -8], [82, -4, -4], [14, 0, 24], [-22, 5, -6], [-76, 2, 6], [44, 0, 2], [18, 6, -2], [-54, -14, 1], [-8, 15, -5], [-20, 11, 5], [18, -23, -7], [30, 8, 4], [10, -14, -2], [-14, 15, 8], [24, 22, -6], [6, 3, 5], [12, 15, 16], [10, 13, -2], [-46, -2, 9], [-30, -19, 0], [16, 2, 12], [50, 8, -12], [20, 12, 0], [46, -17, -15], [28, 8, 6], [18, -15, -12], [2, -20, -2]]
